#version 330 core

in vec2 TexCoords;
out vec4 FragColor;

uniform sampler2D inputTexture; // Textura de entrada
uniform vec2 textureSize; // Tamanho da textura
uniform float blurAmount; // Intensidade do desfoque

// Função para calcular o desfoque gaussiano
float gaussian(float x, float sigma) {
    return exp(-x * x / (2.0 * sigma * sigma)) / (sqrt(2.0 * 3.14159265358979323846) * sigma);
}

void main() {
    vec2 texelSize = 1.0 / textureSize;

    // Núcleo do desfoque gaussiano
    const int kernelSize = 9;
    float kernel[kernelSize];
    float totalWeight = 0.0;
    for (int i = 0; i < kernelSize; ++i) {
        float offset = float(i - (kernelSize - 1) / 2);
        kernel[i] = gaussian(offset, blurAmount);
        totalWeight += kernel[i];
    }
    for (int i = 0; i < kernelSize; ++i) {
        kernel[i] /= totalWeight;
    }

    // Aplica o desfoque gaussiano na textura de entrada
    vec3 result = vec3(0.0);
    for (int x = -kernelSize / 2; x <= kernelSize / 2; ++x) {
        for (int y = -kernelSize / 2; y <= kernelSize / 2; ++y) {
            vec2 offset = vec2(x, y) * texelSize;
            result += texture(inputTexture, TexCoords + offset).rgb * kernel[x + kernelSize / 2] * kernel[y + kernelSize / 2];
        }
    }

    FragColor = vec4(result, 1.0);
}
